﻿using Kivii.Linq;
using Kivii.Testing;
using Kivii.Third.Scjgj.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Third.Scjgj
{
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PlanQuery : RestfulExecution<Plan>
    {
        public int? Skip { get; set; } = 0;

        public int? Take { get; set; } = 50;

        public string OrderBy { get; set; }
        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new RestfulQueryResponse<Plan>();
            rtns.Results = new List<Plan>();
            var conn = KiviiContext.GetOpenedDbConnection<Plan>();
            try
            {
                var query = conn.From<Plan>();
                if (!QueryValues.IsNullOrEmpty()) query.Where(o => o.PLAN_CODE.Contains(QueryValues));
                if (Skip.HasValue && Skip.Value >= 0) query.Skip(Skip.Value);
                if (Take.HasValue && Take.Value > 0) query.Take(Take.Value);
                if (!OrderBy.IsNullOrEmpty()) query.OrderBy(OrderBy);
                rtns.Results = conn.Select(query);


                rtns.Total = conn.Scalar<Plan, int>(o => Sql.Count(o.ID));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PlanDetailQuery : RestfulExecution<PlanDetail>
    {
        public int? Skip { get; set; } = 0;

        public int? Take { get; set; } = 50;

        public string OrderBy { get; set; }
        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        //public bool Convert { get; set; }
        //#region QueryArgs
        //public virtual int? Skip { get; set; }

        //public virtual int? Take { get; set; }

        //public virtual string OrderBy { get; set; }

        //public string OrderByDesc { get; set; }

        //public virtual string Include { get; set; }

        //public virtual string Fields { get; set; }

        //public string QueryKeys { get; set; }

        //public string QueryValues { get; set; }

        //#endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new RestfulQueryResponse<PlanDetail>();
            rtns.Results = new List<PlanDetail>();
            var conn = KiviiContext.GetOpenedDbConnection<Plan>();
            try
            {
                var query = conn.From<PlanDetail>();
                if (!QueryValues.IsNullOrEmpty()) query.Where(o => o.PLAN_CODE.Contains(QueryValues));
                if (Skip.HasValue && Skip.Value >= 0) query.Skip(Skip.Value);
                if (Take.HasValue && Take.Value > 0) query.Take(Take.Value);
                if (!OrderBy.IsNullOrEmpty()) query.OrderBy(OrderBy);
                var results = conn.Select(query);
                int total = 0;
                if (!QueryValues.IsNullOrEmpty()) total = conn.Scalar<PlanDetail, int>(o => Sql.Count(o.ID), o => o.PLAN_CODE.Contains(QueryValues));
                else total = conn.Scalar<PlanDetail, int>(o => Sql.Count(o.ID));
                rtns.Results = results;
                rtns.Total = total;
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    public class PlanDetailConvert : RestfulExecution<PlanDetail>
    {
        public string BatchNumber { get; set; }//计划编号PLAN_CODE

        public string PlanNumber { get; set; }//计划批次号LOT_NUM

        public bool ToReport { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            BatchNumber.ThrowIfNullOrEmpty("请传入计划编号");
            PlanNumber.ThrowIfNullOrEmpty("请传入批次号");
            var result = new PlanDetail();
            var conn = KiviiContext.GetOpenedDbConnection<Plan>();
            try
            {
                var query = conn.From<PlanDetail>();
                query.Where(o => o.PLAN_CODE == BatchNumber && o.LOT_NUM == PlanNumber);
                var planDetail = conn.Single(query);
                if (planDetail == null) throw new Exception("未找到此任务单");
                result = planDetail.DataFilling(conn);

            }
            catch (Exception ex)
            {
                throw ex;
            }

            if (!ToReport)
            {
                var rtns = new RestfulReadResponse<PlanDetail>();
                rtns.Result = result;
                return rtns;
            }
            else
            {
                var rtns = new RestfulReadResponse<Report>();
                rtns.Result = result.ConvertTo();
                return rtns;
            }
        }
    }
}
